import { Router } from "./router";
import { Portfolio } from "./portfolio";

/** The configuration options for a Mosaic component.
 * @typedef {MosaicOptions} MosaicOptions Configuration options for a Mosaic component.
 * @param {HTMLElement} element The DOM element to inject this component into.
 * @param {Object} data The data of this component.
 * @param {Function | String} view The view to define the appearance of this component.
 * @param {Function} created Called when this component is created.
 * @param {Function} willUpdate Called when this component is about to update.
 * @param {Function} updated Called when this component has been updated.
 * @param {Function} willDestroy Called when this component is about to be destroyed. */
export type MosaicOptions = {
    /** The HTML element to inject this Mosaic component into. */
    element: string|HTMLElement,

    /** The state of this component. */
    data: Object,

    /** The view that will be rendered on the screen. */
    view: Function,

    /** A client-side router that allows Mosaics to be used as routes. */
    router?: Router,

    portfolio?: Portfolio,

    /** The actions that can be used on this Mosaic component. */
    actions: Object,

    /** The function to run when this component is created and injected into the DOM. */
    created?: Function,

    /** The function to run when this component is about to update its data. */
    willUpdate?: Function,

    /** The function to run after this component has been updated. */
    updated?: Function,

    /** The function that runs just before this component gets removed from the DOM. */
    willDestroy?: Function,
}